// asio_client.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
#include <random>
#include <climits>
#include <algorithm>
#include <chrono>
#include <thread>
#include <boost/asio.hpp>
#include <boost/make_shared.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

boost::asio::io_service io_service;
boost::asio::io_service::work work(io_service);

#if defined(_MSC_VER)
using asio_tp = boost::asio::ip::tcp;
const asio_tp::endpoint ep(boost::asio::ip::address::from_string("127.0.0.1"), 4444);
#else
using asio_tp = boost::asio::local::stream_protocol;
const std::string uds_path("/Users/Enji/760ed723-d550-45fa-8f54-dde5033ce459");
const asio_tp::endpoint ep(uds_path);
#endif

void do_connect();

void disconnector(boost::shared_ptr<boost::thread> writer_thread, boost::shared_ptr<asio_tp::socket> socket)
{
	// wait arbitary seconds (0-5 sec)
	std::random_device r;
	std::default_random_engine ngine(r());

	std::uniform_int_distribution<int> unif_dist(0, 5000); // 0 - 5sec
	auto wait = unif_dist(ngine);

	std::this_thread::sleep_for(std::chrono::milliseconds(wait));

	writer_thread->interrupt();

	if (socket->is_open())
	{
		socket->cancel();
		socket->close();
	}

	writer_thread->join();

	do_connect();
}

void massive_writer(boost::shared_ptr<asio_tp::socket> socket)
{
	do {
		// alloc
		auto buffer = std::make_shared<std::vector<unsigned char>>(1920 * 1080 * 4);
		assert(buffer != nullptr);

		// make randomize
		std::generate(std::begin(*buffer), std::end(*buffer), []() { return std::rand() % 0xFF; });

		// send
		boost::system::error_code ec;
		std::size_t sent = boost::asio::write(*socket, boost::asio::buffer(*buffer), ec);
		if (ec)
		{
			std::cout << "errored : " << ec.message() << ". in " << __func__ << std::endl;
		}
		else
		{
			std::cout << "sent : " << sent << std::endl;
		}
	} while (socket->is_open());
}

void handle_connect(boost::shared_ptr<asio_tp::socket> socket,
	const boost::system::error_code& ec)
{
	if (ec)
	{
		std::cout << "errored : " << ec.message() << ". in " << __func__ << std::endl;
		return;
	}

	// create send thread 
	auto writer_thread = boost::make_shared<boost::thread>(
		boost::bind(massive_writer, socket));

	// create disconnect timer thread
	boost::thread(boost::bind(disconnector, writer_thread, socket));
}

void do_connect()
{
	auto socket = boost::make_shared<asio_tp::socket>(io_service);
	
	socket->async_connect(
		ep,
		boost::bind(
			handle_connect,
			socket,
			boost::asio::placeholders::error)
	);
}

int main()
{
	boost::thread io_thread([]() {io_service.run(); });

	do_connect();

	io_thread.join();
	return 0;
}

